using System;
using System.Collections.Generic;
using System.Text;

namespace InfoQ.AspectsOfDMM.Version1
{
    /// <summary>
    /// The Employee class, which inherits from the Person class
    /// should support both dirty tracking and lazy loading.
    /// </summary>
    public class Employee : Person, ILazy
    {
        public override string Name
        {
            get
            {
                if (!loaded)
                {
                    ((ILazy)this).Loaded = true;

                    //call the persistence component
                    //and ask it to load the object
                    //with data from the data source
                    //(code omitted for brevity...
                    //simulate by setting field)
                    this.name = "John";
                }
                return base.Name;
            }
            set
            {
                if (!loaded)
                {
                    ((ILazy)this).Loaded = true;

                    //simulate lazy loading...
                    this.name = "John";
                }
                base.Name = value;
            }
        }

        private decimal salary;
        public decimal Salary
        {
            get 
            {
                if (!loaded)
                {
                    ((ILazy)this).Loaded = true;

                    //simulate lazy loading...
                    this.salary = 1000;
                }

                return salary; 
            }
            set
            {
                if (!loaded)
                {
                    ((ILazy)this).Loaded = true;

                    //simulate lazy loading...
                    this.salary = 1000;
                }

                if (value != salary)
                    ((IDirty)this).Dirty = true;

                salary = value;
            }
        }

        private bool loaded;
        /// <summary>
        /// The Loaded property is "write-once" -
        /// after you have set it to true you can not set 
        /// it to false again
        /// </summary>
        bool ILazy.Loaded
        {
            get { return loaded; }
            set
            {
                if (loaded)
                    return;

                Console.WriteLine("[Interceptor lazy loading object]");

                loaded = value;
            }
        }
    }
}
