using System;
using System.Collections.Generic;
using System.Text;
using InfoQ.AspectsOfDMM.Domain;

namespace InfoQ.AspectsOfDMM.Version2
{
    public class EmployeeProxy : Employee, ILazy
    {
        public override string Name
        {
            get 
            {
                if (!loaded)
                {
                    ((ILazy)this).Loaded = true;
                    
                    //call the persistence component
                    //and ask it to load the object
                    //with data from the data source
                    //(code omitted for brevity...
                    //simulate by setting field)
                    this.name = "John";                    
                }
                return base.Name;
            }
            set
            {
                if (!loaded)
                {
                    ((ILazy)this).Loaded = true;

                    //simulate lazy loading...
                    this.name = "John";
                }

                if (value != this.name)
                    ((IDirty)this).Dirty = true;

                base.Name = value;
            }
        }

        public override decimal Salary
        {
            get 
            {
                if (!loaded)
                {
                    ((ILazy)this).Loaded = true;

                    //simulate lazy loading...
                    this.salary = 1000;
                }
                return base.Salary; 
            }
            set
            {
                if (!loaded)
                {
                    ((ILazy)this).Loaded = true;

                    //simulate lazy loading...
                    this.salary = 1000;
                }

                if (value != this.salary)
                    ((IDirty)this).Dirty = true;

                base.Salary = value;
            }
        }

        private bool loaded;
        /// <summary>
        /// The Loaded property is "write-once" -
        /// after you have set it to true you can not set 
        /// it to false again
        /// </summary>
        bool ILazy.Loaded
        {
            get { return loaded; }
            set
            {
                if (loaded)
                    return;

                Console.WriteLine("[Interceptor lazy loading object]");

                loaded = value;
            }
        }
    }
}
