using System;
using System.Collections.Generic;
using System.Text;
using InfoQ.AspectsOfDMM.Domain;

namespace InfoQ.AspectsOfDMM.Version3
{
    public class EmployeeProxy : Employee, ILazy
    {
        //This mixin contains the implementation
        //of the ILazy interface.
        private ILazy lazyMixin = new LazyMixin();

        private LazyInterceptor lazyInterceptor = new LazyInterceptor();
        private DirtyInterceptor dirtyInterceptor = new DirtyInterceptor(); 

        public override string Name
        {
            get
            {
                lazyInterceptor.OnPropertyGetSet(this, "Name");
                return base.Name;
            }
            set
            {
                lazyInterceptor.OnPropertyGetSet(this, "Name");
                dirtyInterceptor.OnPropertySet(this, this.name, value);
                base.Name = value;
            }
        }

        public override decimal Salary
        {
            get
            {
                lazyInterceptor.OnPropertyGetSet(this, "Salary");
                return base.Salary;
            }
            set
            {
                lazyInterceptor.OnPropertyGetSet(this, "Salary");
                dirtyInterceptor.OnPropertySet(this, this.name, value);
                base.Salary = value;
            }
        }

        //The ILazy interface is implemented 
        //by forwarding the calls to the mixin, 
        //which contains the actual implementation.
        bool ILazy.Loaded
        {
            get { return lazyMixin.Loaded; }
            set { lazyMixin.Loaded = value; }
        }
    }
}
